PROGRAM SpaceGroupMultiplication;

USES
 Crt,Printer;

TYPE
 ElementMatType=ARRAY[1..2,1..48,1..3,1..3] OF Integer;
 MultType=ARRAY[1..2,1..48,1..48] OF Integer;
 ElementSymbolType=ARRAY[1..2,1..2,1..48] OF STRING[12];
 TType=ARRAY[1..3] OF Real;

VAR
 ElementMat:ElementMatType;
 Mult:MultType;
 ElementSymbol:ElementSymbolType;
 X1,X2,X3,Y1,Y2,Y3,Z1,Z2,Z3,XFinal,YFinal,ZFinal:Real;
 R1,R2,GroupType,Notation:Integer;
 Option:Char;
 TVar,TVar2:TType;

PROCEDURE Initialize;

VAR
 A,B,C:Integer;
 CubMultFile,HexMultFile,ElementSymbolFile,HexElementMatFile,
 CubElementMatFile,HexElementFile:Text;

BEGIN
 Assign(CubMultFile,'DOUCUBM.DAT');
 Assign(HexMultFile,'DOUHEXM.DAT');
 Assign(ElementSymbolFile,'CUBELSY.DAT');
 Assign(CubElementMatFile,'CUMAT.DAT');
 Assign(HexElementMatFile,'HEMAT.DAT');
 Assign(HexElementFile,'HEXELSY.DAT');
 Reset(CubMultFile);
 Reset(HexMultFile);
 Reset(ElementSymbolFile);
 Reset(CubElementMatFile);
 Reset(HexElementMatFile);
 Reset(HexElementFile);
 FOR A:=1 TO 24 DO
  BEGIN
   FOR B:=1 TO 24 DO
    BEGIN
     Read(CubMultFile,Mult[1,A,B]);
     IF (A<=12) AND (B<=12) THEN
      Read(HexMultFile,Mult[2,A,B]);
    END;
  END;
 FOR A:=1 TO 24 DO
  BEGIN
   FOR B:=1 TO 24 DO
    BEGIN
     IF Mult[1,A,B]>48 THEN
      Mult[1,A,B]:=Mult[1,A,B]-48;
     IF (A<=12) AND (B<=12) THEN
      BEGIN
       IF Mult[2,A,B]>48 THEN
        Mult[2,A,B]:=Mult[2,A,B]-48;
      END;
    END;
  END;
 FOR A:=1 TO 24 DO
  BEGIN
   FOR B:=1 TO 24 DO
    BEGIN
     Mult[1,A+24,B+24]:=Mult[1,A,B];
     Mult[1,A+24,B]:=Mult[1,A,B]+24;
     Mult[1,A,B+24]:=Mult[1,A,B]+24;
     IF (A<=12) AND (B<=12) THEN
      BEGIN
       Mult[2,A+12,B+12]:=Mult[2,A,B];
       Mult[2,A,B+12]:=Mult[2,A,B]+12;
       Mult[2,A+12,B]:=Mult[2,A,B]+12;
      END;
    END;
  END;
 FOR A:=1 TO 2 DO
  BEGIN
   FOR B:=1 TO 48 DO
    BEGIN
     Read(ElementSymbolFile,ElementSymbol[1,A,B]);
     IF (B MOD 6)=0 THEN
      Readln(ElementSymbolFile);
    END;
  END;
 FOR A:=1 TO 2 DO
  BEGIN
   FOR B:=1 TO 24 DO
    BEGIN
     Read(HexElementFile,ElementSymbol[2,A,B]);
     IF (B MOD 6)=0 THEN
      Readln(HexElementFile);
    END;
  END;
 FOR A:=1 TO 48 DO
  BEGIN
   FOR B:=1 TO 3 DO
    BEGIN
     FOR C:=1 TO 3 DO
      BEGIN
       Read(CubElementMatFile,ElementMat[1,A,B,C]);
       IF A<=24 THEN
        Read(HexElementMatFile,ElementMat[2,A,B,C]);
      END;
    END;
   Readln(CubElementMatFile);
   IF A<=24 THEN
    Readln(HexElementMatFile);
  END;
END;

PROCEDURE ChangeNotation;

BEGIN
 IF Notation=1 THEN Notation:=2
 ELSE IF Notation=2 THEN Notation:=1;
END;

PROCEDURE Introduction;

BEGIN
 ClrScr;
 Writeln;
 Writeln('                      Space Multiplication Program');
 Writeln;
 Writeln('                      1) Begin program using Cubic groups');
 Writeln;
 Writeln('                      2) Begin program using Hexagonal groups');
 Writeln;
 IF (Notation=1) THEN
  Writeln('                      3) Change from International to Schoenflies notation')
 ELSE
  Writeln('                      3) Change from Schoenflies to International notation');
 Writeln;
 Writeln('                      0) Exit the program');
 Writeln;
 Write('                                                  Press 1, 2, 3 or 0:');
 Option:=Readkey;
 IF (Option='1') THEN
  GroupType:=1;
 IF (Option='2') THEN
  GroupType:=2;
 IF (Option='3') THEN
  ChangeNotation;
END;

PROCEDURE Calculate;

VAR
 A,B:Integer;

BEGIN
 XFinal:=0;
 YFinal:=0;
 ZFinal:=0;
 TVar[1]:=X1;
 TVar[2]:=Y1;
 TVar[3]:=Z1;
 TVar2[1]:=X2;
 TVar2[2]:=Y2;
 TVar2[3]:=Z2;
 FOR A:=1 TO 3 DO
  BEGIN
   XFinal:=XFinal+ElementMat[GroupType,R1,1,A]*TVar2[A];
   YFinal:=YFinal+ElementMat[GroupType,R1,2,A]*TVar2[A];
   ZFinal:=ZFinal+ElementMat[GroupType,R1,3,A]*TVar2[A];
  END;
 XFinal:=XFinal+TVar[1];
 YFinal:=YFinal+TVar[2];
 ZFinal:=ZFinal+TVar[3];
 ClrScr;
 Writeln;
 Write('{',ElementSymbol[GroupType,Notation,R1],'| ',X1:0:2,', ',Y1:0:2,', ',Z1:0:2,'}');
 Writeln('*{',ElementSymbol[GroupType,Notation,R2],'| ',X2:0:2,', ',Y2:0:2,', ',Z2:0:2,'} = ');
 Writeln;
 Writeln('{',ElementSymbol[GroupType,Notation,Mult[GroupType,R1,R2]],'| ',XFinal:0:2,', ',YFinal:0:2,', ',ZFinal:0:2,'}');
 Writeln;
 Writeln;
 Writeln;
 Writeln;
 Write('Press enter to continue: ');
 Readln;
END;


PROCEDURE Menu;

VAR
 A,Code:Integer;
 Options:Char;

BEGIN
 REPEAT
  ClrScr;
  Writeln;
  Writeln('(R1|t1) = (R1|(t1(x),t1(y),t2(z)))');
  Writeln('(R2|t2) = (R2|(t2(x),t2(y),t2(z)))');
  Writeln;
  Writeln;
  IF (GroupType=1) THEN
  BEGIN
   FOR A:=1 TO 16 DO
    BEGIN
     Writeln(A:2,') ',ElementSymbol[GroupType,Notation,A],'        ',A+16:2,') ',
             ElementSymbol[GroupType,Notation,A+16],'        ',A+32:2,') ',
             ElementSymbol[GroupType,Notation,A+32]);
    END;
  END
  ELSE IF (GroupType=2) THEN
  BEGIN
   FOR A:=1 TO 12 DO
    BEGIN
     Writeln(A:2,') ',ElementSymbol[GroupType,Notation,A],'        ',A+12:2,') ',
             ElementSymbol[GroupType,Notation,A+12]);
    END;
  END;
  Writeln;
  Write('Choose an element R1 and press enter: ');
  Readln(R1);
 UNTIL (R1>=1)AND(R1<=48);
 REPEAT
  ClrScr;
  Writeln;
  Writeln('(R1|t1) = (',ElementSymbol[GroupType,Notation,R1],'|t1(X),t1(Y),t1(Z))');
  Writeln('(R2|t2) = (R2|t2(X),t2(Y),t2(Z))');
  Writeln;
  Writeln;
  Writeln('Enter t1(X), t1(Y), and t1(Z)');
  Readln(X1,Y1,Z1);
 UNTIL ((X1>=-32768)AND(X1<=32767)AND(Y1>=-32768)AND(Y1<=32767)AND(Z1>=-32768)AND(Z1<=32767));
 REPEAT
  ClrScr;
  Writeln;
  Writeln('(R1|t1) = (',ElementSymbol[GroupType,Notation,R1],'|',X1:0:2,',',Y1:0:2,',',Z1:0:2,')');
  Writeln('(R2|t2) = (R2|(t2(x),t2(y),t2(z))');
  Writeln;
  Writeln;
  IF (GroupType=1) THEN
   BEGIN
    FOR A:=1 TO 16 DO
     BEGIN
      Writeln(A:2,') ',ElementSymbol[GroupType,Notation,A],'        ',A+16:2,') ',
              ElementSymbol[GroupType,Notation,A+16],'        ',A+32:2,') ',
              ElementSymbol[GroupType,Notation,A+32]);
     END;
   END
   ELSE IF (GroupType=2) THEN
   BEGIN
    FOR A:=1 TO 12 DO
     BEGIN
      Writeln(A:2,') ',ElementSymbol[GroupType,Notation,A],'        ',A+12:2,') ',
              ElementSymbol[GroupType,Notation,A+12]);
     END;
   END;
  Writeln;
  Writeln('Choose an element R2 and press enter: ');
  Readln(R2);
 UNTIL (R2>=1)AND(R2<=48);
 REPEAT
  ClrScr;
  Writeln;
  Writeln('(R1|t1) = (',ElementSymbol[GroupType,Notation,R1],'|',X1:0:2,',',Y1:0:2,',',Z1:0:2,')');
  Writeln('(R2|t2) = (',ElementSymbol[GroupType,Notation,R2],'|t2(X),t2(Y),t2(Z))');
  Writeln;
  Writeln;
  Writeln('Enter t2(X), t2(Y), and t2(Z)');
  Readln(X2,Y2,Z2);
 UNTIL ((X2>=-32768)AND(X2<=32767)AND(Y2>=-32768)AND(Y2<=32767)AND(Z2>=-32768)AND(Z2<=32767));
 Calculate;
END;

BEGIN
 Notation:=1;
 GroupType:=1;
 Initialize;
 REPEAT
  Introduction;
  IF ((Option='1')OR(Option='2')) THEN Menu;
 UNTIL (Option='0');
END.